CWE = {
    "Access of Resource Using Incompatible Type ('Type Confusion')" : [843],
    "Access of Uninitialized Pointer": [824],
    "Allocation of Resources Without Limits or Throttling": [770],
    "Authentication Errors": [1211, 288, 290, 294, 295, 296, 299, 303, 304, 305, 306, 307, 308, 309, 322, 603, 645, 804, 846],
    "Authentication Issues": [1211, 288, 290, 294, 295, 296, 299, 303, 304, 305, 306, 307, 308, 309, 322, 603, 645, 804, 846],
    "Buffer Access with Incorrect Length Value": [805],
    "Buffer Copy without Checking Size of Input ('Classic Buffer Overflow')": [120],
    "Buffer Errors": [1218, 120, 123, 124, 125, 131, 786, 787, 788, 805],
    "Cleartext Transmission of Sensitive Information": [319],
    "Code Injection": [94, 96],
    "Code": [94, 96, 393, 394, 215, 540, 470, 1265],
    "Command Injection": [77, 78],
    "Concurrent Execution using Shared Resource with Improper Synchronization ('Race Condition')": [362],
    "Configuration": [16],
    "Credentials Management": [255, 256, 257, 260, 261, 262, 263, 324, 521, 523, 549, 620, 640, 798, 916],
    "Cryptographic Issues": [310, 261, 324, 325, 328, 331, 334, 335, 338, 347, 916, 1240],
    "DEPRECATED: Information Exposure Through Debug Log Files": [534],
    "Data Handling": [19, 130, 166, 167, 168, 178, 182, 186, 229, 233, 237, 241, 409, 471, 472, 601, 611, 624, 625, 776, 1024],
    "Data Processing Errors": [19, 130, 166, 167, 168, 178, 182, 186, 229, 233, 237, 241, 409, 471, 472, 601, 611, 624, 625, 776, 1024],
    "Deadlock": [833],
    "Divide By Zero": [369],
    "Double Free": [415],
    "Error Handling": [728, 7, 203, 209, 228, 252, 289, 390, 391, 394, 636],
    "Excessive Iteration": [834],
    "Expected Behavior Violation": [440],
    "Exposure of Resource to Wrong Sphere": [668],
    "Exposure of Sensitive Information to an Unauthorized Actor": [200],
    "Format String Vulnerability": [133, 134, 135, 597],
    "Heap-based Buffer Overflow": [122],
    "Improper Access Control": [284],
    "Improper Authentication": [287],
    "Improper Authorization": [285],
    "Improper Check for Unusual or Exceptional Conditions": [754],
    "Improper Control of Dynamically-Managed Code Resources": [913],
    "Improper Control of Generation of Code ('Code Injection')": [94],
    "Improper Error Handling": [728, 7, 203, 209, 228, 252, 289, 390, 391, 394, 636],
    "Improper Handling of Exceptional Conditions": [755],
    "Improper Initialization": [665],
    "Improper Input Validation": [20],
    "Improper Limitation of a Pathname to a Restricted Directory ('Path Traversal')": [22],
    "Improper Locking": [667],
    "Improper Neutralization of Special Elements in Output Used by a Downstream Component ('Injection')": [74],
    "Improper Neutralization of Special Elements used in a Command ('Command Injection')": [77],
    "Improper Preservation of Permissions": [281],
    "Improper Privilege Management": [269],
    "Improper Resource Shutdown or Release": [404],
    "Improper Restriction of Communication Channel to Intended Endpoints": [923],
    "Improper Restriction of Excessive Authentication Attempts": [307],
    "Improper Restriction of Operations within the Bounds of a Memory Buffer": [119],
    "Improper Validation of Array Index": [129],
    "Improper Validation of Integrity Check Value": [354],
    "Improper Verification of Cryptographic Signature": [347],
    "Improperly Implemented Security Check for Standard": [358],
    "Inadequate Encryption Strength": [326],
    "Incomplete Cleanup": [459],
    "Incorrect Authorization": [863],
    "Incorrect Calculation of Buffer Size": [131],
    "Incorrect Calculation": [682],
    "Incorrect Comparison": [697],
    "Incorrect Conversion between Numeric Types": [681],
    "Incorrect Default Permissions": [276],
    "Incorrect Permission Assignment for Critical Resource": [732],
    "Incorrect Type Conversion or Cast": [704],
    "Information Exposure Through Log Files": [532],
    "Information Exposure": [200, 201, 203, 209, 213, 215, 359, 497, 538, 1243, 1258, 1273, 1295],
    "Information Leak / Disclosure": [200, 201, 203, 209, 213, 215, 359, 497, 538, 1243, 1258, 1273, 1295],
    "Injection": [1026, 1027, 77, 78, 88, 917, 943],
    "Input Validation": [1019, 20, 59, 74, 75, 76, 77, 78, 88, 94, 96, 99, 138, 349, 502, 641, 790, 791, 795],
    "Insecure Default Initialization of Resource": [1188],
    "Insertion of Sensitive Information into Externally-Accessible File or Directory": [538],
    "Insufficient Information": [778, 223, 221, 222],
    "Insufficient Verification of Data Authenticity": [345],
    "Integer Overflow or Wraparound": [190],
    "Integer Underflow (Wrap or Wraparound)": [191],
    "Interpretation Conflict": [436],
    "Link Following": [59, 61, 62],
    "Loop with Unreachable Exit Condition ('Infinite Loop')": [835],
    "Memory Buffer Errors": [1218, 120, 123, 124, 125, 131, 786, 787, 788, 805],
    "Missing Authentication for Critical Function": [306],
    "Missing Authorization": [862, 638, 1314],
    "Missing Initialization of Resource": [909],
    "Missing Release of Memory after Effective Lifetime": [401],
    "Missing Release of Resource after Effective Lifetime": [772],
    "NULL Pointer Dereference": [476],
    "Non-exit on Failed Initialization": [455],
    "Numeric Errors": [189, 128, 190, 191, 192, 193, 197, 198, 369, 681, 839, 1077],
    "Observable Differences in Behavior to Error Inputs": [203, 205],
    "Observable Discrepancy": [203],
    "Off-by-one Error": [193],
    "Operation on a Resource after Expiration or Release": [672],
    "Other": [],
    "Out-of-bounds Read": [125],
    "Out-of-bounds Write": [787],
    "Path Traversal": [22, 23, 36],
    "Permissions, Privileges, and Access Control": [264],
    "Permissions, Privileges, and Access Controls": [264],
    "Privilege Dropping / Lowering Errors": [271],
    "Race Conditions": [362, 364, 366, 367, 368, 421, 689, 1223, 1298],
    "Release of Invalid Pointer or Reference": [763],
    "Resource Management Errors": [399, 73, 403, 410, 470, 502, 619, 641, 694, 763, 770, 771, 772, 826, 908, 909, 910, 911, 914, 915, 920, 1188],
    "Security Features": [254, 256, 258, 259, 260, 261, 272, 284, 285, 330, 359, 798],
    "Stack-based Buffer Overflow": [121],
    "String Errors": [133, 134, 135, 597],
    "Time-of-check Time-of-use (TOCTOU) Race Condition": [367],
    "Unchecked Return Value": [252],
    "Uncontrolled Resource Consumption ('Resource Exhaustion')": [400],
    "Uncontrolled Resource Consumption": [400],
    "Unspecified": [],
    "Use After Free": [416],
    "Use of Insufficiently Random Values": [330],
    "Use of Uninitialized Resource": [908],
    "Use of a Broken or Risky Cryptographic Algorithm": [327],
}
